/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright held by original author
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "incrementTractionDisplacementFvPatchVectorField.H"
#include "addToRunTimeSelectionTable.H"
#include "volFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

incrementTractionDisplacementFvPatchVectorField::
incrementTractionDisplacementFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedGradientFvPatchVectorField(p, iF),
    incrementTraction_(p.size(), vector::zero),
    incrementPressure_(p.size(), 0.0)
{
    fvPatchVectorField::operator=(patchInternalField());
    gradient() = vector::zero;
}


incrementTractionDisplacementFvPatchVectorField::
incrementTractionDisplacementFvPatchVectorField
(
    const incrementTractionDisplacementFvPatchVectorField& tdpvf,
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedGradientFvPatchVectorField(tdpvf, p, iF, mapper),
    incrementTraction_(tdpvf.incrementTraction_, mapper),
    incrementPressure_(tdpvf.incrementPressure_, mapper)
{}


incrementTractionDisplacementFvPatchVectorField::
incrementTractionDisplacementFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const dictionary& dict
)
:
    fixedGradientFvPatchVectorField(p, iF),
    incrementTraction_("incrementTraction", dict, p.size()),
    incrementPressure_("incrementPressure", dict, p.size())
{
    fvPatchVectorField::operator=(patchInternalField());
    gradient() = vector::zero;
}


incrementTractionDisplacementFvPatchVectorField::
incrementTractionDisplacementFvPatchVectorField
(
    const incrementTractionDisplacementFvPatchVectorField& tdpvf
)
:
    fixedGradientFvPatchVectorField(tdpvf),
    incrementTraction_(tdpvf.incrementTraction_),
    incrementPressure_(tdpvf.incrementPressure_)
{}


incrementTractionDisplacementFvPatchVectorField::
incrementTractionDisplacementFvPatchVectorField
(
    const incrementTractionDisplacementFvPatchVectorField& tdpvf,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedGradientFvPatchVectorField(tdpvf, iF),
    incrementTraction_(tdpvf.incrementTraction_),
    incrementPressure_(tdpvf.incrementPressure_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void incrementTractionDisplacementFvPatchVectorField::autoMap
(
    const fvPatchFieldMapper& m
)
{
    fixedGradientFvPatchVectorField::autoMap(m);
    incrementTraction_.autoMap(m);
    incrementPressure_.autoMap(m);
}


// Reverse-map the given fvPatchField onto this fvPatchField
void incrementTractionDisplacementFvPatchVectorField::rmap
(
    const fvPatchVectorField& ptf,
    const labelList& addr
)
{
    fixedGradientFvPatchVectorField::rmap(ptf, addr);

    const incrementTractionDisplacementFvPatchVectorField& dmptf =
        refCast<const incrementTractionDisplacementFvPatchVectorField>(ptf);

    incrementTraction_.rmap(dmptf.incrementTraction_, addr);
    incrementPressure_.rmap(dmptf.incrementPressure_, addr);
}


// Update the coefficients associated with the patch field
void incrementTractionDisplacementFvPatchVectorField::updateCoeffs()
{
    if (updated())
    {
        return;
    }
   const dictionary& mechanicalProperties =
        db().lookupObject<IOdictionary>("mechanicalProperties");

    dimensionedScalar E(mechanicalProperties.lookup("E"));
    dimensionedScalar nu(mechanicalProperties.lookup("nu"));

    dimensionedScalar mu = E/(2.0*(1.0 + nu));
    dimensionedScalar lambda = nu*E/((1.0 + nu)*(1.0 - 2.0*nu));

    Switch planeStress(mechanicalProperties.lookup("planeStress"));

    if (planeStress)
    {
        lambda = nu*E/((1.0 + nu)*(1.0 - nu));
    }

    vectorField n = patch().nf();

    const fvPatchField<tensor>& graddU =
        patch().lookupPatchField<volTensorField, tensor>("grad(dU)");

    const fvPatchField<tensor>& deps_p =
        patch().lookupPatchField<volTensorField, tensor>("deps_p");

    gradient() =
    (
        (incrementTraction_ - incrementPressure_*n)
      - (n & (mu.value()*graddU.T() - (mu + lambda).value()*graddU))
      - n*tr(graddU)*lambda.value()
      + (n & (2*mu.value()*deps_p))
      + n*tr(deps_p)*lambda.value()
    )/(2.0*mu + lambda).value();

    fixedGradientFvPatchVectorField::updateCoeffs();
}


// Write
void incrementTractionDisplacementFvPatchVectorField::write(Ostream& os) const
{
    fvPatchVectorField::write(os);
    incrementTraction_.writeEntry("incrementTraction", os);
    incrementPressure_.writeEntry("incrementPressure", os);
    writeEntry("value", os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

makePatchTypeField(fvPatchVectorField, incrementTractionDisplacementFvPatchVectorField);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
